import java.io.*;
import java.nio.channels.*;

class FileUtils {
    public static final String BACKUP_EXT = ".backup";

    /**
     * ñΥե򥳥ԡ
     * @param src	ԡե
     * @param dst	ԡե(ǥ쥯ȥϻԲ)
     * @param preserve	ǽݻ뤫Υե饰
     */
    public static void copyFile(File src, File dst, boolean preserve) throws IOException {
	FileChannel srcChan = new FileInputStream(src).getChannel();
	FileChannel destChan = new FileOutputStream(dst).getChannel();
	srcChan.transferTo(0, srcChan.size(), destChan);
	srcChan.close();
	destChan.close();

	// ǽ򥳥ԡ
	if(preserve)
	    dst.setLastModified(src.lastModified());
    }

    /**
     * եƵŪ˥ԡ
     * @param src	ԡ
     * @param dst	ԡ
     * @param preserve	ǽݻ뤫Υե饰
     */
    public static void copyRecursive(File src, File dst, boolean preserve) throws IOException {
	if(src.isFile()) {
	    copyFile(src, dst, preserve);
	    return;
	}

	if(src.isDirectory()) {
	    String[] entries = src.list();
	    for(int i = 0; i < entries.length; i++) {
		File src2 = new File(src, entries[i]);
		File dst2 = new File(dst, entries[i]);
		if(src2.isFile())
		    copyFile(src2, dst2, preserve);
		else if(src2.isDirectory())
		    copyRecursive(src2, dst2, preserve);
		else
		    throw new RuntimeException(src2.getPath()
					       + " is not file nor directory");
	    }
	}

	throw new RuntimeException(src.getPath()
				   + " is not file nor directory");
    }

    /**
     * ǽݻƥХååפ
     */
    public static void makeBackupFile(File file) throws IOException {
	makeBackupFile(file, true);
    }

    /**
     * եΥХååפ
     */
    public static void makeBackupFile(File file, boolean preserve) throws IOException {
	if(!file.exists())
	    return; // ꥸʥ뤬¸ߤʤʤ鲿⤷ʤ

	File backup = new File(file.getPath() + BACKUP_EXT);
	if(backup.exists())
	    return; // Хååפ¸ߤʤ鲿⤷ʤ

	copyFile(file, backup, preserve);
    }

    /**
     * եڤͤ롣
     * λˤޤХååפƤʤä顢Хååפ롣
     */
    public static void truncate(File file) throws IOException {
	File backup = new File(file.getPath() + BACKUP_EXT);
	if(backup.isFile()) {
	    file.delete();
	    touch(file);
	} else {
	    file.renameTo(backup);
	    touch(file);
	}
    }

    /**
     * եκǽ򹹿롣ե뤬¸ߤʤϺ롣
     */
    public static void touch(File file) throws IOException {
	if(!file.exists())
	    (new FileOutputStream(file)).close();
	if(!file.isFile())
	    return; // Ρå
	file.setLastModified((new java.util.Date()).getTime());
    }
}
